/**
 * Configuration options for encryption/decryption
 */
export interface CryptoConfig {
    /** Maximum allowed text length for encryption (default: 10000) */
    maxTextLength?: number;
    /** Custom encryption key (if not provided, uses AES_SECRET_KEY env var) */
    encryptionKey?: string;
    /** Algorithm to use (default: aes-256-cbc) */
    algorithm?: string;
}
/**
 * Result of encryption operation
 */
export interface EncryptionResult {
    /** Base64 encoded encrypted data */
    encryptedData: string;
    /** Base64 encoded initialization vector */
    iv: string;
    /** Optional authentication tag for GCM modes */
    authTag?: string;
}
/**
 * Custom error classes for better error handling
 */
export declare class CryptoError extends Error {
    code: string;
    constructor(message: string, code: string);
}
export declare class ValidationError extends CryptoError {
    constructor(message: string);
}
export declare class DecryptionError extends CryptoError {
    constructor(message: string);
}
/**
 * Encrypts text using AES-256-CBC with a random IV
 *
 * @param text - The text to encrypt
 * @param config - Optional configuration
 * @returns Object containing encrypted data and IV (both base64 encoded)
 *
 * @example
 * ```typescript
 * const result = encrypt("Hello World");
 * console.log(result.encryptedData); // base64 encoded encrypted data
 * console.log(result.iv); // base64 encoded IV
 * ```
 */
export declare function encrypt(text: string, config?: CryptoConfig): EncryptionResult;
/**
 * Decrypts data that was encrypted with the encrypt function
 *
 * @param encryptedData - Base64 encoded encrypted data
 * @param iv - Base64 encoded initialization vector
 * @param config - Optional configuration
 * @returns The decrypted text
 *
 * @example
 * ```typescript
 * const decrypted = decrypt(result.encryptedData, result.iv);
 * console.log(decrypted); // "Hello World"
 * ```
 */
export declare function decrypt(encryptedData: string, iv: string, config?: CryptoConfig): string;
/**
 * Generates a secure random key suitable for AES encryption
 *
 * @param length - Key length in bytes (default: 32 for AES-256)
 * @returns Base64 encoded random key
 *
 * @example
 * ```typescript
 * const key = generateKey();
 * process.env.AES_SECRET_KEY = key;
 * ```
 */
export declare function generateKey(length?: number): string;
/**
 * Securely compares two strings to prevent timing attacks
 *
 * @param a - First string
 * @param b - Second string
 * @returns True if strings are equal
 */
export declare function safeCompare(a: string, b: string): boolean;
//# sourceMappingURL=index.d.ts.map