"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DecryptionError = exports.ValidationError = exports.CryptoError = void 0;
exports.encrypt = encrypt;
exports.decrypt = decrypt;
exports.generateKey = generateKey;
exports.safeCompare = safeCompare;
const crypto_1 = require("crypto");
/**
 * Custom error classes for better error handling
 */
class CryptoError extends Error {
    constructor(message, code) {
        super(message);
        this.code = code;
        this.name = "CryptoError";
    }
}
exports.CryptoError = CryptoError;
class ValidationError extends CryptoError {
    constructor(message) {
        super(message, "VALIDATION_ERROR");
    }
}
exports.ValidationError = ValidationError;
class DecryptionError extends CryptoError {
    constructor(message) {
        super(message, "DECRYPTION_ERROR");
    }
}
exports.DecryptionError = DecryptionError;
/**
 * Validates if a string is valid base64
 */
function isValidBase64(str) {
    // Check if string contains only valid base64 characters
    const base64Regex = /^[A-Za-z0-9+/]*={0,2}$/;
    if (!base64Regex.test(str)) {
        return false;
    }
    // Check if length is valid (must be multiple of 4 when padded)
    const paddedLength = str.length + ((4 - (str.length % 4)) % 4);
    if (paddedLength % 4 !== 0) {
        return false;
    }
    try {
        // Try to decode and re-encode to verify it's valid base64
        const decoded = Buffer.from(str, "base64");
        const reencoded = decoded.toString("base64");
        // Remove any padding from original for comparison
        const normalizedOriginal = str.replace(/=+$/, "");
        const normalizedReencoded = reencoded.replace(/=+$/, "");
        return normalizedOriginal === normalizedReencoded;
    }
    catch {
        return false;
    }
}
/**
 * Validates and sanitizes the encryption key
 */
function getEncryptionKey(customKey) {
    const key = customKey || process.env.AES_SECRET_KEY;
    if (!key) {
        throw new ValidationError("Encryption key not provided. Set AES_SECRET_KEY environment variable or pass key in config.");
    }
    if (key.length < 32) {
        throw new ValidationError("Encryption key must be at least 32 characters long.");
    }
    // Use exactly 32 bytes for AES-256
    return Buffer.from(key.slice(0, 32), "utf8");
}
/**
 * Validates input text for encryption
 */
function validateTextInput(text, maxLength) {
    if (typeof text !== "string") {
        throw new ValidationError("Input must be a string.");
    }
    if (text.length === 0) {
        throw new ValidationError("Input text cannot be empty.");
    }
    if (text.length > maxLength) {
        throw new ValidationError(`Input text exceeds maximum length of ${maxLength} characters.`);
    }
}
/**
 * Validates encrypted data and IV for decryption
 */
function validateDecryptionInputs(encryptedData, iv) {
    if (typeof encryptedData !== "string" || !encryptedData) {
        throw new ValidationError("Encrypted data must be a non-empty string.");
    }
    if (typeof iv !== "string" || !iv) {
        throw new ValidationError("IV must be a non-empty string.");
    }
    if (!isValidBase64(encryptedData)) {
        throw new ValidationError("Invalid encrypted data format. Expected base64 encoded string.");
    }
    if (!isValidBase64(iv)) {
        throw new ValidationError("Invalid IV format. Expected base64 encoded string.");
    }
    const ivBuffer = Buffer.from(iv, "base64");
    if (ivBuffer.length !== 16) {
        throw new ValidationError("Invalid IV length. Expected 16 bytes.");
    }
}
/**
 * Encrypts text using AES-256-CBC with a random IV
 *
 * @param text - The text to encrypt
 * @param config - Optional configuration
 * @returns Object containing encrypted data and IV (both base64 encoded)
 *
 * @example
 * ```typescript
 * const result = encrypt("Hello World");
 * console.log(result.encryptedData); // base64 encoded encrypted data
 * console.log(result.iv); // base64 encoded IV
 * ```
 */
function encrypt(text, config = {}) {
    const { maxTextLength = 10000, encryptionKey: customKey, algorithm = "aes-256-cbc", } = config;
    try {
        validateTextInput(text, maxTextLength);
        const encryptionKey = getEncryptionKey(customKey);
        const IV_LENGTH = 16;
        const iv = (0, crypto_1.randomBytes)(IV_LENGTH);
        const cipher = (0, crypto_1.createCipheriv)(algorithm, encryptionKey, iv);
        let encrypted = cipher.update(text, "utf8", "base64");
        encrypted += cipher.final("base64");
        return {
            encryptedData: encrypted,
            iv: iv.toString("base64"),
        };
    }
    catch (error) {
        if (error instanceof CryptoError) {
            throw error;
        }
        throw new CryptoError(`Encryption failed: ${error instanceof Error ? error.message : "Unknown error"}`, "ENCRYPTION_ERROR");
    }
}
/**
 * Decrypts data that was encrypted with the encrypt function
 *
 * @param encryptedData - Base64 encoded encrypted data
 * @param iv - Base64 encoded initialization vector
 * @param config - Optional configuration
 * @returns The decrypted text
 *
 * @example
 * ```typescript
 * const decrypted = decrypt(result.encryptedData, result.iv);
 * console.log(decrypted); // "Hello World"
 * ```
 */
function decrypt(encryptedData, iv, config = {}) {
    const { encryptionKey: customKey, algorithm = "aes-256-cbc" } = config;
    try {
        validateDecryptionInputs(encryptedData, iv);
        const encryptionKey = getEncryptionKey(customKey);
        const ivBuffer = Buffer.from(iv, "base64");
        const decipher = (0, crypto_1.createDecipheriv)(algorithm, encryptionKey, ivBuffer);
        let decrypted = decipher.update(encryptedData, "base64", "utf8");
        decrypted += decipher.final("utf8");
        return decrypted;
    }
    catch (error) {
        if (error instanceof CryptoError) {
            throw error;
        }
        throw new DecryptionError(`Decryption failed: ${error instanceof Error ? error.message : "Unknown error"}`);
    }
}
/**
 * Generates a secure random key suitable for AES encryption
 *
 * @param length - Key length in bytes (default: 32 for AES-256)
 * @returns Base64 encoded random key
 *
 * @example
 * ```typescript
 * const key = generateKey();
 * process.env.AES_SECRET_KEY = key;
 * ```
 */
function generateKey(length = 32) {
    if (length < 16) {
        throw new ValidationError("Key length must be at least 16 bytes.");
    }
    return (0, crypto_1.randomBytes)(length).toString("base64");
}
/**
 * Securely compares two strings to prevent timing attacks
 *
 * @param a - First string
 * @param b - Second string
 * @returns True if strings are equal
 */
function safeCompare(a, b) {
    if (typeof a !== "string" || typeof b !== "string") {
        return false;
    }
    const bufferA = Buffer.from(a, "utf8");
    const bufferB = Buffer.from(b, "utf8");
    if (bufferA.length !== bufferB.length) {
        return false;
    }
    return (0, crypto_1.timingSafeEqual)(bufferA, bufferB);
}
//# sourceMappingURL=index.js.map